Object.defineProperty(exports, "__esModule", { value: true });
const core_1 = require("@babel/core");
const core_2 = require("@webpackery/core");
const lodash_1 = require("lodash");
const tsconfig_paths_webpack_plugin_1 = require("tsconfig-paths-webpack-plugin");
const inject_empty_exports_1 = require("./inject-empty-exports");
const ts_utils_1 = require("./ts-utils");
const omitKeys = [
    "extensions",
    "tsconfigPath",
    "useReact",
    "useTypesScript",
];
class BabelConfigurer extends core_2.WebpackConfigurer {

    prepare(props, context) {
        const { production } = context;

        props = Object.assign({}, props);
        props.options = Object.assign({ sourceMaps: production, envName: production ? "production" : "development" }, props.options);

        const loaded = core_1.loadOptions(Object.assign({ filename: __filename }, props.options));

        core_2.setDefaultBy(props, "useReact", isReact, loaded.plugins);
        core_2.setDefaultBy(props, "useTypesScript", isTypeScript, loaded.plugins);
        core_2.setDefaultBy(props, "extensions", resolveDefaultExtensions, props);
        core_2.setDefaultBy(props, "test", extensionsToRegex, props.extensions);
        return props;
    }

    configure(context, props) {
        const custom = {
            plugins: [],
        };

        if (props.useTypesScript) {
            const ts = require("typescript");
            const tsconfigPath = props.tsconfigPath || ts_utils_1.findTsConfigFile(ts, context.config.context);
            if (tsconfigPath) {
                const tsconfig = ts_utils_1.readTsConfigFile(ts, tsconfigPath);
                const { compilerOptions } = tsconfig;
                const { paths, preserveConstEnums } = compilerOptions;

                if (paths)
                    context.resolve.plugin(new tsconfig_paths_webpack_plugin_1.TsconfigPathsPlugin({
                        extensions: props.extensions,
                        configFile: tsconfigPath,
                    }));

                custom.plugins.push(constEnumPlugin(preserveConstEnums), inject_empty_exports_1.InjectEmptyExports);
            }
        }
        const rule = Object.assign(Object.assign({ loader: require.resolve("./babel-custom-loader") }, lodash_1.omit(props, omitKeys)), { options: Object.assign({ custom }, props.options) });
        context.module.rule(rule);
        context.resolve.extension(...props.extensions);
    }
}
exports.BabelConfigurer = BabelConfigurer;
function extensionsToRegex(extensions) {
    if (extensions && extensions.length) {
        const group = extensions.map(x => x.slice(1)).join("|");
        const pattern = `\.(${group})$`;
        return new RegExp(pattern);
    }
}
function resolveDefaultExtensions(props) {
    const { useReact, useTypesScript } = props;
    return [
        ".js", ".es6", ".es", ".mjs",
        useReact && ".jsx",
        useTypesScript && [".ts", ".d.ts"],
        useTypesScript && useReact && ".tsx",
    ].filter(Boolean).flat();
}
function isReact(plugins) {
    return plugins.some(isReactPlugin);
}
function isTypeScript(plugins) {
    return plugins.some(isTypeScriptPlugin);
}

function isTypeScriptPlugin(plugin) {
    return plugin.key.startsWith("transform-typescript");
}

function isReactPlugin(plugin) {
    return plugin.key.startsWith("transform-react");
}

function constEnumPlugin(preserveConstEnums) {
    const transform = preserveConstEnums ? "removeConst" : "constObject";
    return ["babel-plugin-const-enum", { transform }];
}
//# sourceMappingURL=babel-configurer.js.map
